<?php declare(strict_types=1);

namespace App\Form;

use App\Entity\MenuItem;
use App\Repository\MenuItemRepository;
use Doctrine\ORM\QueryBuilder;
use Override;
use Symfony\Bridge\Doctrine\Form\Type\EntityType;
use Symfony\Bridge\Doctrine\Types\UlidType;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Component\Uid\Ulid;

use function array_map;
use function assert;

final class MenuItemType extends AbstractType
{
    #[Override]
    public function buildForm(FormBuilderInterface $builder, array $options): void
    {
        $item = $options['data'];
        assert($item instanceof MenuItem); // Ensure it's of the correct type
        $vendorId = $item->getFoodVendor()?->getId(); // Use safe navigation operator in case FoodVendor is null

        $builder->add('name'); // Basic field
        $builder->add('aliases', EntityType::class, [
            'class' => MenuItem::class,
            'choice_label' => 'name',
            'multiple' => true,
            'expanded' => true,
            'query_builder' => static function (MenuItemRepository $repository) use ($item, $vendorId): QueryBuilder {

                $ids = $repository->createQueryBuilder('m')
                    ->select('DISTINCT IDENTITY(m.aliasOf)')
                    ->where('m.deletedAt IS NULL')
                    ->andWhere('m.aliasOf IS NOT NULL')
                    ->getquery();
                $ids = $ids->getScalarResult();
                $ids = array_map(static fn(array $id): Ulid => Ulid::fromBinary($id[1]), $ids);

                // Build the main query with a NOT EXISTS constraint
                $qb = $repository->createQueryBuilder('m');
                $qb
                    ->where('m.foodVendor = :vendorId')
                    ->andWhere('m.deletedAt IS NULL')
                    ->andWhere('m.id != :id');
                foreach ($ids as $key => $id) {
                    $qb->andWhere("m.id != :idBy{$key}");
                    $qb->setParameter("idBy{$key}", $id, UlidType::NAME);
                }
                $qb
                    ->orderBy('m.name', 'ASC')
                    ->setParameter('vendorId', $vendorId, UlidType::NAME) // ULID or appropriate type
                    ->setParameter('id', $item->getId()); // ULID or appropriate type
                return $qb;
            },
        ]);
    }

    #[Override]
    public function configureOptions(OptionsResolver $resolver): void
    {
        $resolver->setDefaults([
            'data_class' => MenuItem::class,
        ]);
    }
}
